#!/usr/local/bin/perl
#-------------------------------------------
# MiniSearch v0.1
# Personal Website Search Engine
# http://www.dansteinman.com/search/
#
# Copyright (C) 1999 Dan Steinman
# Distributed under the terms of the of the GNU General Public License
# You may modify this file, redistribute it, but any significant improvements must be made public
# Read the LICENSE file for more details
#-------------------------------------------
# search.pl
# the "crawler" that shows the results of the search
#-------------------------------------------

$root = "$ENV{'DOCUMENT_ROOT'}";					# finds the path to your files
$datadir = "$root/cgi-bin/searchdata";			# directory to store data
$webaddress = "http://www.dansteinman.com";	# url to show

&ReadParse;

print "Content-type: text/html\n\n";

#-------------------------------------------
# decide what to do

$qstr = $ENV{'QUERY_STRING'};

if ($qstr eq "") {
	&displayPrompt;
}
else {
	&displayResults;
}

#-------------------------------------------
# write the search prompt page
sub displayPrompt {
	print <<"EOF"

<html>
<title>Search</title>
<body bgcolor="#FFFFFF">

<h3>Search</h3>

<form action="search.pl" method="get">	

<table cellspacing=0 cellpadding=2>
<tr>
<td>Search for:</td>
<td><input name="query" type="text" size=20></td>
</tr>

<tr>
<td>Match:</td>
<td><select name="match">
<option value="all" selected>all words
<option value="any">any words
</select></td>
</tr>

<tr>
<td>Number of results</td>
<td><select name="max">
<option value="10" selected>10 per page
<option value="20">20 per page
<option value="unlimited">unlimited
</select></td>
</tr>

<tr><td><input type="submit" value="Search"></td></tr>
</table>

</form>
	
</body>
</html>

EOF

} # end sub printPrompt

#-------------------------------------------
# show the results of the search
sub displayResults {

if ($in{'query'} eq "") {
	&displayPrompt;
	exit(0);
}
$in{'query'} =~ s/'//g;
@queries = split(/ /,$in{'query'});
$totalqueries = @queries;
$match = ($in{'match'} || "all");
$set = ($in{'set'} || 1);
$max = ($in{'max'} || 10);

open(FILE,"$datadir/files");
while ($line = <FILE>) {
	@parts = split(/,/,$line);
	push(@files,$parts[1]);
	$file = $parts[1];
	$matches{$file} = 0;
	$sizes{$file} = $parts[2];
	$dates{$file} = $parts[3];
	$titles{$file} = $parts[4];
}
close(FILE);

open(FILE,"$datadir/words");
while (<FILE>) {
	next unless s/^(.*?) = \s*//;
	$words{$1} = [ split ];
}
close(FILE);

foreach $query (@queries) {
	if (@{$words{$query}}) {
		foreach $index (@{$words{$query}}) {
			$potmatches{$files[$index]} += 1;
		}
	}
}

foreach $file (keys(%potmatches)) {
	next if ($match eq "any" && $potmatches{$file} == 0);
	next if ($match eq "all" && $potmatches{$file} < $totalqueries);
	push(@matches,$file);
}

$nummatches = @matches;
if ($in{'max'} eq 'unlimited') {$max = $nummatches;}

print <<"EOF";

<html>
<title>Search Results for: $in{'query'}</title>
<body bgcolor="#FFFFFF">

<h3>Search Results</h3>

EOF

if ($nummatches > 0) {
	print "$nummatches matches were found:";

	$lastmatch = $max+($set-1)*$max;
	$lastmatch = $nummatches if ($lastmatch>$nummatches);
	
	for ($i=($set-1)*$max;$i<$lastmatch;$i++) {
		$file = $matches[$i];
		$ip1 = $i+1;
		my($ksize) = ($sizes{$file}/1000);
		$size = &round($ksize,1);
		$date = &formattedDate($dates{$file});
		print <<"EOF";
	
		<p><b>$ip1.</b> <a href="$file">$titles{$file}</a>
		<br><b>URL:</b> $webaddress$file
		<br><b>Size:</b> ${size}KB, <b>Date:</b> $date

EOF
	}
	$numsets = int($nummatches/$max)+1;
	$next = $set+1;
	$qstr =~ s/\&set=(.*[0-9])//;
	$qstr = "search.pl?$qstr&set=";
}
if ($nummatches == 0) {
	print "<p>Sorry, no matches found.";
}

print <<"EOF";

<form action="search.pl" method="get">		
<td><input name="query" type="text" size=20 value="$in{'query'}"></td>
<input type="hidden" name="match" value="$in{'match'}">
<input type="hidden" name="max" value="$in{'max'}">
<input type="submit" value="Search">
<a href="search.pl">new search</a>
	
EOF
if ($numsets>1) {
	print "<p>More: ";
	
	if ($numsets>15) {$showsets = 15;}
	else {$showsets = $numsets;}
	for $i (1..$showsets) {
		if ($i == $set) {print "$i ";}
		else {print "<a href=\"$qstr$i\">$i</a> ";}
	}
	if ($set<$numsets) {
		print "| <a href=\"$qstr$next\">Next</a>";
	}
}

print <<"EOF"

<p><font size=2><a href="http://www.dansteinman.com/search/">MiniSearch v0.1</a></font>
</body>
</html>

EOF

} # end sub displayResults


#-------------------------------------------
# some helper routines

# get the input
sub ReadParse {
local (*in) = @_ if @_;
local ($i, $loc, $key, $val);
if ($ENV{'REQUEST_METHOD'} eq "GET") {
	$in = $ENV{'QUERY_STRING'};
} elsif ($ENV{'REQUEST_METHOD'} eq "POST") {
	read(STDIN,$in,$ENV{'CONTENT_LENGTH'});
}
@in = split(/&/,$in);
foreach $i (0 .. $#in) {
	$in[$i] =~ s/\+/ /g;
	($key, $val) = split(/=/,$in[$i],2);
	$key =~ s/%(..)/pack("c",hex($1))/ge;
	$val =~ s/%(..)/pack("c",hex($1))/ge;
	$in{$key} .= "\0" if (defined($in{$key}));
	$in{$key} .= $val;
}
return 1;
} # end sub ReadParse

# round $num to $dec decimal places
sub round {
	my($num,$dec) = @_;
	my($exp) = 10**$dec;
	return(int($num*$exp+$exp/20)/$exp);
} # end sub dec

# make it look nicer
sub formattedDate {
	my($date) = @_;
	($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime($date);
	@months = ('Jan','Feb','Mar','Apr','May','Jun','Jul','Aug','Sep','Oct','Nov','Dec');
	if ($year<100) {$year += 1900;}
	$fdate = "$months[$mon] $mday, $year";
	return $fdate;
} # end sub formatedDate
